/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.0
 *
 * @date        2022-09-30
 *
 * @attention
 *
 *  Copyright (C) 2022-2024 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be useful and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"
#include "bsp_delay.h"
#include "apm32f035_m0cp.h"
#include "apm32f035_rcm.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

/** @addtogroup Examples
  * @brief M0CP CORDIC vector examples
  @{
  */

/** @addtogroup M0CP_VectorCORDIC_X
  @{
  */

/** @defgroup M0CP_VectorCORDIC_X_Macros Macros
  @{
*/

/* printf using USART1 */
#define DEBUG_USART             USART1
/* the result amplitude */
#define AMP 1000
/**@} end of group M0CP_VectorCORDIC_X_Macros*/

/** @defgroup M0CP_VectorCORDIC_X_Variables Variables
  @{
  */
/* test array */
int16_t data_x[4] = {1, 2, 3, 6};
int16_t data_y[4] = {1, 1, 4, 8};
/**@} end of group M0CP_VectorCORDIC_X_Variables*/

/** @defgroup M0CP_VectorCORDIC_X_Functions Functions
  @{
  */

void APM_MINI_Init(void);
void M0CP_Init(void);

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 *
 */
int main(void)
{
    int16_t x = 0;
    int16_t y = 0;
    int16_t z = 0;
    /* input maximum value */
    int16_t MAX = (int16_t)sqrt(pow((1.0 * 32767 / 1.648 / AMP), 2) / 2);
    uint16_t i = 0;
    int16_t resultTemp[3];

    APM_MINI_Init();

    M0CP_Init();

    printf("This is an example of M0CP CORDIC vector operation \r\n");

    printf("2 second to test random data .\r\n");

    printf("You can press KEY1 to start test specified data array.\r\n");

    APM_MINI_DelayMs(2000);

    while (1)
    {
        /* random input  */
        x = rand() % MAX + 1;
        y = rand() % MAX + 1;
        /* calculate the test array */
        if (APM_MINI_PBGetState(BUTTON_KEY1) == 0)
        {
            printf("calculate the test array\r\n");
            for (i = 0; i < 4; i++)
            {
                /* CORDIC vector algorithm of X*/
                /* X = Sqrt(X^2+Y^2)*/
                /* Prepare operation data */
#if M0CP_USE_FAST_IQ
                M0CP_Fast_IQVectorCORDIC(data_x[i] * AMP, data_y[i] * AMP, z, resultTemp);

                /* data conversion */
                printf("CORDIC_Vector(%d,%d,%d) = %0.3f\r\n",
                       data_x[i], data_y[i], z, 1.0 * resultTemp[M0CP_X_INDEX] / 1.648 / AMP);
#else
                if (M0CP_IQVectorCORDIC(data_x[i] * AMP, data_y[i] * AMP, z, resultTemp) != M0CP_OK)
                {
                    printf("M0CP CORDIC vector timeout\r\n");
                }
                else
                {
                    /* data conversion */
                    printf("CORDIC_Vector(%d,%d,%d) = %0.3f\r\n",
                           data_x[i], data_y[i], z, 1.0 * resultTemp[M0CP_X_INDEX] / 1.648 / AMP);
                }
#endif

                APM_MINI_DelayMs(500);
            }
            printf("finish calculate\r\n");
            while (APM_MINI_PBGetState(BUTTON_KEY1) == 0);
        }

        /* calculate the random data */
#if M0CP_USE_FAST_IQ
        M0CP_Fast_IQVectorCORDIC(x * AMP, y * AMP, z, resultTemp);

        /* data conversion */
        printf("CORDIC_Vector(%d,%d,%d) = %0.3f\r\n",
               x, y, z, 1.0 * resultTemp[M0CP_X_INDEX] / 1.648 / AMP);
#else
        if (M0CP_IQVectorCORDIC(x * AMP, y * AMP, z, resultTemp) != M0CP_OK)
        {
            printf("M0CP CORDIC vector timeout\r\n");
        }
        else
        {
            /* data conversion */
            printf("CORDIC_Vector(%d,%d,%d) = %0.3f\r\n",
                   x, y, z, 1.0 * resultTemp[M0CP_X_INDEX] / 1.648 / AMP);
        }
#endif

        APM_MINI_LEDToggle(LED2);
        APM_MINI_DelayMs(500);
    }
}

/*!
 * @brief       Init MINI board
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void APM_MINI_Init(void)
{
    /* Init delay function */
    APM_MINI_DelayInit();

    APM_MINI_LEDInit(LED2);
    APM_MINI_LEDInit(LED3);
    APM_MINI_PBInit(BUTTON_KEY1, BUTTON_MODE_GPIO);
    APM_MINI_COMInit(COM1);
}

/*!
 * @brief       M0CP initialization
 *
 * @param       None
 *
 * @retval      None
 *
 */
void M0CP_Init(void)
{
    M0CP_Config_T m0cpConfigStruct;

    /* Load firmware from Flash to SRAM */
    M0CP_Firmware_Init();

    /* Reset M0CP */
    RCM_EnableAHBPeriphReset(RCM_AHB_PERIPH_M0CP);
    RCM_DisableAHBPeriphReset(RCM_AHB_PERIPH_M0CP);

    /* Enable M0CP clock */
    RCM_EnableAHBPeriphClock(RCM_AHB_PERIPH_M0CP);

    /* Init M0CP hardware */
    M0CP_HardInit();

    M0CP_ClearStatusFlag(M0CP_FLAG_DONE);
    /* Disable M0CP TX interrupt */
    M0CP_DisableInterrupt(M0CP_INT_TX);

    /* Wait for M0CP to be ready */
    while (M0CP_ReadStatusFlag(M0CP_FLAG_BUSY) == SET)
    {

    }

    M0CP_ConfigStructInit(&m0cpConfigStruct);

    /* Configure M0CP */
    m0cpConfigStruct.algebraMode        = M0CP_ALG_ATAN2;
    m0cpConfigStruct.bankAutoConv       = M0CP_BANK_CONV_SOFTWARE;
    m0cpConfigStruct.bankRegGroup       = M0CP_BANK_REG_GROUP_0;
    m0cpConfigStruct.fixedPoint         = M0CP_FP_Q15;
    m0cpConfigStruct.xyResultCORDIC     = M0CP_CORDIC_XY;
    M0CP_Config(&m0cpConfigStruct);
}

#if defined (__CC_ARM) || defined (__ICCARM__) || (defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050))

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       ch:  The characters that need to be send.
*
* @param       *f:  pointer to a FILE that can recording all information
*              needed to control a stream
*
* @retval      The characters that need to be send.
*
* @note
*/
int fputc(int ch, FILE* f)
{
    /* send a byte of data to the serial port */
    USART_TxData(DEBUG_USART, (uint8_t)ch);

    /* wait for the data to be send  */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return (ch);
}

#elif defined (__GNUC__)

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       ch:  The characters that need to be send.
*
* @retval      The characters that need to be send.
*
* @note
*/
int __io_putchar(int ch)
{
    /* send a byte of data to the serial port */
    USART_TxData(DEBUG_USART, ch);

    /* wait for the data to be send  */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return ch;
}

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       file:  Meaningless in this function.
*
* @param       *ptr:  Buffer pointer for data to be sent.
*
* @param       len:  Length of data to be sent.
*
* @retval      The characters that need to be send.
*
* @note
*/
int _write(int file, char* ptr, int len)
{
    int i;
    for (i = 0; i < len; i++)
    {
        __io_putchar(*ptr++);
    }

    return len;
}

#else
#warning Not supported compiler type
#endif

/**@} end of group M0CP_VectorCORDIC_X_Functions */
/**@} end of group M0CP_VectorCORDIC_X */
/**@} end of group Examples */
